<?php
/*
* 2007-2016 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2016 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_'))
exit;

class FattorinoIt extends CarrierModule
{
	public  $id_carrier;

	public function __construct()
	{
		$this->name = 'fattorinoit';
		$this->tab = 'front_office_features';
		$this->version = '1.0';
		$this->author = 'Fattorino.it';
		$this->need_instance = 0;

		$this->bootstrap = true;
		parent::__construct();

		$this->displayName = $this->l('Fattorino.it - subito da te');
		$this->description = $this->l('Aggiunge metodo di spedizione Fattorino.it');
		$this->ps_versions_compliancy = [
			'min' => '1.7.1.0',
			'max' => _PS_VERSION_,
		];
		$this->confirmUninstall = $this->l('Vuoi rimuovere fattorino.it?');

		if (self::isInstalled($this->name)) {
			# prendo lista dei corrieri già installati
			global $cookie;
			$carriers = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
			$id_carrier_list = array();
			foreach($carriers as $carrier)
			$id_carrier_list[] .= $carrier['id_carrier'];


			$warning = array();
			if (!in_array((int)(Configuration::get('MYCARRIER1_CARRIER_ID')), $id_carrier_list))
			$warning[] .= $this->l('"Carrier 1"').' ';
			if (count($warning))
			$this->warning .= implode(' , ',$warning).$this->l('must be <br />configured');
		}
	}

	public function install()
	{
		$this->_clearCache('*');

		$carrierConfig = array(
			0 => array(
				'name' => 'Fattorino.it',
				'id_tax_rules_group' => 0, #nessuna tassa
				'active' => true,
				'deleted' => 0,
				'shipping_handling' => false,
				'range_behavior' => 0,
				'delay' => array(
					Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => 'Description 1'
				),
				'id_zone' => 1,
				'id_shop' => Context::getContext()->shop->id,
				'is_module' => true,
				'shipping_external' => true,
				'external_module_name' => $this->name, // We specify the name of the module
				'need_range' => true
			),
		);

		$id_carrier = $this->installExternalCarrier($carrierConfig[0]);


		Configuration::updateValue('FATTORINO_CARRIER_ID', $id_carrier);
		Configuration::updateValue('FATTORINO_ENABLE_FOR', 0);
		Configuration::updateValue('FATTORINO_AVAILABLE_PROVINCE', "");
		Configuration::updateValue('FATTORINO_AVAILABLE_CAP', "");
		Configuration::updateValue('FATTORINO_SHIPPING_OFFSET', 0);
		Configuration::updateValue('FATTORINO_SHIPPING_PRICE', 0);

		$return = $this->callFattorino(json_encode(array('shop' => Context::getContext()->shop, 'url'=>_PS_BASE_URL_, 'version' => _PS_VERSION_, 'brand'=>'ps')), 'install');
		$return_json = json_decode($return);

		if (!parent::install()
		|| !$this->registerHook('displayCarrierExtraContent')
		|| !$this->registerHook('header')

		|| !$this->registerHook('updateCarrier')
		|| !$this->registerHook('orderConfirmation')
		)
		return false;

		return true;
	}

	public static function installExternalCarrier($config)
	{
		$carrier = new Carrier();
		$carrier->name = $config['name'];
		$carrier->id_tax_rules_group = $config['id_tax_rules_group'];
		$carrier->id_zone = $config['id_zone'];
		$carrier->active = $config['active'];
		$carrier->deleted = $config['deleted'];
		$carrier->delay = $config['delay'];
		$carrier->shipping_handling = $config['shipping_handling'];
		$carrier->range_behavior = $config['range_behavior'];
		$carrier->is_module = $config['is_module'];
		$carrier->shipping_external = $config['shipping_external'];
		$carrier->external_module_name = $config['external_module_name'];
		$carrier->need_range = $config['need_range'];

		$languages = Language::getLanguages(true);
		foreach ($languages as $language)
		{
			if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
			$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
		}

		if ($carrier->add())
		{
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
			Db::getInstance()->insert('carrier_group', array('id_carrier' => (int)($carrier->id), 'id_group' => (int)($group['id_group'])));

			$rangePrice = new RangePrice();
			$rangePrice->id_carrier = $carrier->id;
			$rangePrice->delimiter1 = '0';
			$rangePrice->delimiter2 = '10000';
			$rangePrice->add();

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone)
			{
				Db::getInstance()->insert('carrier_zone', array('id_carrier' => (int)($carrier->id), 'id_zone' => (int)($zone['id_zone'])));
				Db::getInstance()->insert('delivery', array('id_shop'=> (int)(Context::getContext()->shop->id),'id_carrier' => (int)($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int)($rangeWeight->id), 'id_zone' => (int)($zone['id_zone']), 'price' => '0'));
			}
			}

			if (!copy(dirname(__FILE__).'/carrier.jpg', _PS_SHIP_IMG_DIR_.'/'.(int)$carrier->id.'.jpg'))
			return false;

			return (int)($carrier->id);


		return false;
	}

	public function hookDisplayHeader($params)
	{
		$_controller = Context::getContext()->controller;
		$_controller->addJS(__DIR__.'/js/datepicker.min.js');
		$_controller->addJS(__DIR__.'/js/jquery.cooki-plugin.js');
		$_controller->addJS(__DIR__.'/js/fattorinoit-front.js');
		$_controller->addJS(__DIR__.'/js/moment.min.js');
		$_controller->addJS(__DIR__.'/js/i18n/datepicker.it.js');
		$_controller->addCSS(__DIR__.'/css/datepicker.min.css');
		$_controller->addCSS(__DIR__.'/css/fattorinoit.css');
	}

	public function setMedia()
	{
		$_controller = Context::getContext()->controller;
		$_controller->addJqueryUi('ui.widget');
		$_controller->addJqueryPlugin('tagify');
		$_controller->addJS(__DIR__.'/js/fattorinoit.js');
		$_controller->addCSS(__DIR__.'/css/fattorinoit.css');
	}



	public function renderForm()
	{
		$this->setMedia();
		$return = $this->callFattorino(json_encode(array('shop'=>Context::getContext()->shop)), 'get-cap-by-province');
		if ($return_json = json_decode($return, true)) {
			$province = array_map(function($prov){ return array('id'=>$prov['id'], 'descrizione_provincia'=>$prov['descrizione_provincia']);}, $return_json['data']);
		}

		$return = $this->callFattorino(json_encode(array('shop'=>Context::getContext()->shop)), 'get-post-codes');
		if ($return_json = json_decode($return, true)) {
			$post_codes = array_map(function($pc){ return array('id'=>$pc['id'], 'cap'=>$pc['cap']);}, $return_json['data']);
		}

		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'description' => $this->l('Per comunicare con Fattorino.it è necessario configurare le seguenti informazioni'),
				'input' => array(
					array(
						'type'=> 'html',
						'name'=>'html',
						'html_content'=>'<img src=\''.Tools::getHttpHost(true).__PS_BASE_URI__.'/modules/fattorinoit/carrier.jpg\'><br><br>'
					),
					array(
						'type' => 'select',
						'label' => $this->l('Limita per'),
						'name' => 'FATTORINO_ENABLE_FOR',
						'options' => array(
							'query' => array(
								array(
									'id_option' =>0,
									'name' => 'Non limitare'
								),
								array(
									'id_option' => 1,
									'name' => 'Provincia'
								),
								array(
									'id_option' => 2,
									'name' => 'CAP'
								),
							),
							'id' => 'id_option',
							'name' => 'name'
						),
						'hint' => array(
							$this->l('To add "tags" click in the field, write something and then press "Enter".'),
							$this->l('Forbidden characters:').' &lt;&gt;;=#{}'
							)
						),
						array(
							'type' => 'select',
							'label' => $this->l('Province'),
							'name' => 'FATTORINO_AVAILABLE_PROVINCE',
							'multiple'=>true,
							'class'=>'chosen fixed-width-lg',
							'options' => array(
								'query' => $province,
								'id' => 'id',
								'name' => 'descrizione_provincia'
							),
						),
						array(
							'type' => 'select',
							'label' => $this->l('CAP Abilitati'),
							'name' => 'FATTORINO_AVAILABLE_CAP',
							'multiple'=>true,
							'class'=>'chosen fixed-width-lg',
							'options' => array(
								'query' => $post_codes,
								'id' => 'id',
								'name' => 'cap'
							),
						),


					),
					'submit' => array(
						'title' => $this->l('Save'),
					)
				)
			);

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$this->fields_form = array();
		$helper->id = (int)Tools::getValue('id_carrier');
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitFattorinoIt';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		$helper->tpl_vars['fields_value']['FATTORINO_AVAILABLE_PROVINCE[]'] = explode(";",Configuration::get('FATTORINO_AVAILABLE_PROVINCE'));
		$helper->tpl_vars['fields_value']['FATTORINO_AVAILABLE_CAP[]'] = explode(";",Configuration::get('FATTORINO_AVAILABLE_CAP'));

		return $helper->generateForm(array($fields_form));
	}

	public function getContent()
	{
		$output = '';
		$errors = array();
		if (Tools::isSubmit('submitFattorinoIt'))
		{
			if (Tools::getValue('FATTORINO_ENABLE_FOR') == 1) {
				$province = implode(";", Tools::getValue('FATTORINO_AVAILABLE_PROVINCE'));

				if ($province == null)
				$errors[] = $this->l('Selezionare almeno una provincia');
				else {
					Configuration::updateValue('FATTORINO_AVAILABLE_CAP', null);
				}
			} elseif (Tools::getValue('FATTORINO_ENABLE_FOR') == 2) {
				$cap =   implode(";", Tools::getValue('FATTORINO_AVAILABLE_CAP'));
				if ($cap == null)
				$errors[] = $this->l('Specificare almeno un cap');
				else {
					Configuration::updateValue('FATTORINO_AVAILABLE_PROVINCE', null);
				}
			}


			else {
				$carrier = new Carrier(Configuration::get('FATTORINO_CARRIER_ID'));
				$zones = Zone::getZones(true);
				foreach ($zones as $zone)
				{
					Db::getInstance()->update('delivery', array('price' => $fattorino_shipping_price ), 'id_carrier='.(int)($carrier->id)." and id_zone = ".$zone['id_zone']."");
				}
			}

			if (isset($errors) && count($errors))
			$output = $this->displayError(implode('<br />', $errors));
			else
			{
				Configuration::updateValue('FATTORINO_ENABLE_FOR', Tools::getValue('FATTORINO_ENABLE_FOR'));
				Configuration::updateValue('FATTORINO_SHIPPING_OFFSET', $fattorino_shipping_offset);
				Configuration::updateValue('FATTORINO_SHIPPING_PRICE', $fattorino_shipping_price);
				Configuration::updateValue('FATTORINO_AVAILABLE_PROVINCE', $province);
				Configuration::updateValue('FATTORINO_AVAILABLE_CAP', $cap);
				$output = $this->displayConfirmation($this->l('Impostazioni salvate correttamente.'));
			}
		}
		return $output.$this->renderForm();
	}

	public function getConfigFieldsValues()
	{
		return array(
			'FATTORINO_KEY' => Tools::getValue('FATTORINO_KEY', Configuration::get('FATTORINO_KEY')),
			'FATTORINO_ENABLE_FOR' => Tools::getValue('FATTORINO_ENABLE_FOR', Configuration::get('FATTORINO_ENABLE_FOR')),
			'FATTORINO_AVAILABLE_PROVINCE' => Configuration::get('FATTORINO_AVAILABLE_PROVINCE'),
			'FATTORINO_AVAILABLE_CAP' => Tools::getValue('FATTORINO_AVAILABLE_CAP', Configuration::get('FATTORINO_AVAILABLE_CAP')),
			'FATTORINO_SHIPPING_OFFSET' => Tools::getValue('FATTORINO_SHIPPING_OFFSET', Configuration::get('FATTORINO_SHIPPING_OFFSET')),
			'FATTORINO_SHIPPING_PRICE' => Tools::getValue('FATTORINO_SHIPPING_PRICE', Configuration::get('FATTORINO_SHIPPING_PRICE')),
		);
	}

	public function hookUpdateCarrier($params)
	{
		if ((int)($params['id_carrier']) == (int)(Configuration::get('FATTORINO_CARRIER_ID')))
		Configuration::updateValue('FATTORINO_CARRIER_ID', (int)($params['carrier']->id));
	}

	public function getOrderShippingCost($params, $shipping_cost)	{
		return $this->checkShippingRequirements($params, $shipping_cost);
	}

	public function getOrderShippingCostExternal($params)
	{
		return $this->checkShippingRequirements($params, $shipping_cost);
	}


	private function checkShippingRequirements($params, $shipping_cost) {
		$address = new Address($params->id_address_delivery);
		switch(Configuration::get('FATTORINO_ENABLE_FOR')) {
			case "0":
			return $this->getRates($address);
			break;

			case "1":
			$province = explode(";", Configuration::get('FATTORINO_AVAILABLE_PROVINCE'));
			$return = $this->callFattorino(json_encode(array('shop'=>Context::getContext()->shop, 'province'=> $province)), 'get-cap-by-provincia-id');
			if ($return_json = json_decode($return, true)) {
				$post_codes = array_map(function($pc){ return $pc['cap'];}, $return_json['data']);
			}
			if (in_array($address->postcode, $post_codes )) {
				return $this->getRates($address);
			} else {
				return false;
			}
			break;

			case "2":
			if (in_array($address->postcode, explode(";", Configuration::get('FATTORINO_AVAILABLE_CAP') ))){
				return $this->getRates($address);
			} else {
				return false;
			}
			break;
		}
	}

	private function getRates($address) {
		$return = $this->callFattorino(json_encode(array('shop'=>Context::getContext()->shop, 'postcode'=>$address->postcode, 'fitship'=>$_COOKIE['fitship'])), 'get-rate-post-codes');
		if ($return_json = json_decode($return, true)) {
			if (isset($return_json["data"])){
				return  $return_json["data"]['price'] ;
			}else {
				return false;
			}

		}
	}

	public function hookDisplayOrderConfirmation($params){

		$cntx = Context::getContext();
		$order = $params['order'];
		$customer = new Customer($order->id_customer);
		$address = new Address($order->id_address_delivery);

		$customer_thread = new CustomerThread();
		$customer_thread->id_order = $order->id;
		$customer_thread->id_customer = $order->id_customer;
		$customer_thread->status = 'open';
		$customer_thread->id_shop = $cntx->shop->id;
		$customer_thread->id_lang = 1;
		$customer_thread->id_contact = 0;
		$customer_thread->token = Tools::getToken(false);
		$customer_thread->save();

		$customer_message = new CustomerMessage();
		$customer_message->id_customer_thread = $customer_thread->id;
		$customer_message->id_order = $order->id;
		$customer_message->private = true;
		$customer_message->message = "Fattorino.it-\n Spedizione programmata \n\n";
		$customer_message->save();

		$fitship = "";
		if (isset($_COOKIE['fitship'])) {
			$fitship = $_COOKIE['fitship'];
			unset($_COOKIE['fitship']);
			setcookie('fitship', null, -1, '/');
		}


		$fields = json_encode(array(
			"order_id"=> $order->id,
			"customer_email"=> $customer->email,
			"customer_username"=> $address->lastname." ".$address->firstname,
			"customer_phone"=> $address->phone,
			"customer_address"=> $address->address1 ." - ". $address->postcode." ".$address->city,
			"fitship"=>$fitship,
			"shop" => array(
				"id"=> $cntx->shop->id,
				"name"=> $cntx->shop->name,
				"domain"=> $cntx->shop->domain,
				"domain_ssl"=> $cntx->shop->domain_ssl,
				"url"=> _PS_BASE_URL_
			)
		));


		$return = $this->callFattorino($fields, 'create_task_delivery');
		$return_json = json_decode($return);
		if ($return_json->status == 200) {
			return true;
		}
	}


	private function callFattorino($fields, $action)
	{
		$url = 'https://api.fattorino.it/api/v2/'.$action;
		//$url = 'http://api.fattorino.ahlab/api/v2/'.$action;
		$json = json_encode($fields);



		$cu = curl_init($url);
		curl_setopt($cu, CURLOPT_POST, 1);
		curl_setopt($cu, CURLOPT_VERBOSE, TRUE);
		curl_setopt($cu, CURLOPT_HTTPHEADER, array("fitaccess: 6aa425e73761777a7f9e07e773d7762a"));
		curl_setopt($cu, CURLOPT_POSTFIELDS, "data=".$fields);
		curl_setopt($cu, CURLOPT_RETURNTRANSFER, true);
		$result = curl_exec($cu);


		curl_close($cu);
		return $result;
	}

	public function uninstall()
	{
		$this->_clearCache('*');
		$Carrier1 = new Carrier((int)(Configuration::get('FATTORINO_CARRIER_ID')));

		if (!parent::uninstall() ||
		!Configuration::deleteByName('FATTORINO_CARRIER_ID') ||
		!Configuration::deleteByName('FATTORINO_ENABLE_FOR') ||
		!Configuration::deleteByName('FATTORINO_AVAILABLE_PROVINCE') ||
		!Configuration::deleteByName('FATTORINO_AVAILABLE_CAP') ||
		!Configuration::deleteByName('FATTORINO_SHIPPING_OFFSET') ||
		!Configuration::deleteByName('FATTORINO_SHIPPING_PRICE') ||
		!$this->unregisterHook('updateCarrier'))
		return false;

		if (Configuration::get('PS_CARRIER_DEFAULT') == (int)($Carrier1->id))	{
			global $cookie;

			$carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL);
			foreach($carriersD as $carrierD)
			if ($carrierD['active'] && !$carrierD['deleted'] && ($carrierD['name'] != $this->_config['name']))
			Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
		}

		$Carrier1->deleted = 1;
		if (!$Carrier1->update())
		return false;

		$return = $this->callFattorino(json_encode(array('shop' => Context::getContext()->shop, 'url'=>_PS_BASE_URL_, 'version' => _PS_VERSION_, 'brand'=>'ps')), 'uninstall');
		$return_json = json_decode($return);

		return true;
	}

	public function hookDisplayCarrierExtraContent($params){


		$address = new Address($params['cart']->id_address_delivery);
		$this->smarty->assign(
			array(
				'img' => Tools::getHttpHost(true).__PS_BASE_URI__.'/modules/fattorinoit/carrier.jpg',
				'showFattorino' => 'block',
				'fitcid' => (int)(Configuration::get('FATTORINO_CARRIER_ID')),
				'checkPrice' => array('data' => json_encode(array('shop'=>Context::getContext()->shop, 'postcode'=>$address->postcode)), 'func'=>'get-all-rate-post-codes')
			)
		);
		return $this->display(__FILE__, 'fattorinoit.tpl');


	}

	public function _clearCache($template, $cache_id = NULL, $compile_id = NULL)
	{
		parent::_clearCache('fattorinoit.tpl');
	}
}
